{% extends "default_template.html" %}

{% block menu %}
{% endblock %}

{% block body %}
<h1>Login</h1>

<div class="col-sm-4" id="content">
    <form action="{{ url_for('_login_attempt') }}" method="post">
        <fieldset>
            <div class="form-group">
                <label class="col-form-label" for="inputDefault">Client</label>
                <input type="text" class="form-control" id="client_id" name="client_id">
            </div>
            <div class="form-group">
                <label class="col-form-label" for="inputDefault">Username</label>
                <input type="text" class="form-control" id="username" name="username">
            </div>
            <div class="form-group">
                <label class="col-form-label" for="inputDefault">Password</label>
                <input type="password" class="form-control" id="password" name="password">
            </div>
            <div class="form-check">
                <label class="form-check-label">
                  <input class="form-check-input" type="checkbox" value="X" checked="" id="remember" name="remember">
                  Remember me
                </label>
            </div>
            <br><br>
            <button type="submit" class="btn btn-primary">Login</button>
        </fieldset>
    </form>
</div>
<br><br>
<p class="text-danger"><small><b>This website uses cookies </b> to remember your username. By using this website, you accept our cookie policy.</small></p>
{% endblock %}